==> qmpdclient-1.0.6.2.tar.gz: qmpdclient-1.0.6.2/src/tagguesser.cpp <==
/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "mpdsong.h"
#include "tagguesser.h"
#include <QStringList>

// #define DEBUG(A) A
#define DEBUG(A)

QHash<QString, QString> TagGuesser::patterns() {
	QHash<QString, QString> patterns;
	patterns["/%b/%a/%n - %t."] = "/music/Artist/Album/42 - Title of Song.ogg";
	patterns["/%b/%a (%i)/%n - %t."] = "/music/Artist/Album (1992)/42 - Title of Song.ogg";
	patterns["/%b/%n - %a - %t."] = "/music/Artist/42 - Album - Title of Song.ogg";
	patterns["/%n - %b - %a - %t."] = "/music/42 - Artist - Album - Title of Song.ogg";
	return patterns;
}

void TagGuesser::guessTags(MPDSong *song) {
	QString pattern = Config::instance()->guessPattern();

	// Check that we have a pattern at all
	if (pattern.isEmpty())
		return;

	// Cant guess from URL's
	if (song->url().contains("://"))
		return;

	// Return if we already have everything we can guess
	if (!(song->title().isEmpty() || song->album().isEmpty() || song->artist().isEmpty() || song->track().isEmpty()))
		return;

	DEBUG(qDebug("Guessing.."));
	// Tokens to expand. %a=album, %b=band, %n=tracknr, %t=title, %i=ignore
	QStringList matchers = pattern.split(QRegExp("%[abnti]"), QString::SkipEmptyParts);
	QStringList markers = pattern.split('%', QString::SkipEmptyParts);

	// Check that we actually have any matchers, if not, return filename as title, the rest empty.

	DEBUG(qDebug("Matchers size: %d %d", matchers.size(), markers.size()));
	if (matchers.isEmpty() || markers.isEmpty()) {

		DEBUG(qDebug("No matchers or no markers"));
		if (song->title().isEmpty())
			song->setTitle(song->url());
		return;
	}

	// Remove bogus markers

	DEBUG(qDebug("Removing bogus markers"));
	markers = markers.filter(QRegExp("^[abnti]"));
	// Remove trailing garbage on markers

	DEBUG(qDebug("Removing trailing garbage on markers"));
	markers = markers.replaceInStrings(QRegExp("^([abnti]).*"), "%\\1");


	DEBUG(qDebug("Adding greedy front matcher"));
	// If pattern does not have leading characters, add a greedy matcher at front so we get the whole start
	if (pattern.contains(QRegExp("^%[abnti]")))
		matchers.push_front("^");

	// If pattern contains trailing characters, add a dummy marker at the end.
	DEBUG(qDebug("Finding last matcher"));
	QString lastMarker = markers.last();

	DEBUG(qDebug("Finding end of last marker"));
	int endOfLastMatcher = pattern.lastIndexOf(lastMarker) + lastMarker.size();

	DEBUG(qDebug("Adding dummy end marker"));
	if (endOfLastMatcher < pattern.size())
		markers += "trailingcharacters";

	DEBUG(foreach(QString p, markers) qDebug("marker : %s", qPrintable(p))
	      foreach(QString p, matchers) qDebug("matcher: %s", qPrintable(p)));

	DEBUG(qDebug("checkpoint"));
	// Match pattern, starting last
	QString left = song->url();

	DEBUG(qDebug("Matching pattern, starting last"));
	while (!matchers.empty()) {
		QString p = matchers.takeLast();
		QString m = markers.takeLast();
		int pos = left.lastIndexOf(p);
		QString result = left.mid(pos + p.size());
		DEBUG(qDebug("%s, %s matches: %s at pos %d", qPrintable(left), qPrintable(m), qPrintable(result), pos));
		left = left.mid(0, pos);
		// Save what we found
		if (m == "%a" && song->album().isEmpty())
			song->setAlbum(result);
		if (m == "%b" && song->artist().isEmpty())
			song->setArtist(result);
		if (m == "%n" && song->track().isEmpty())
			song->setTrack(result);
		if (m == "%t" && song->title().isEmpty())
			song->setTitle(result);
	}
}

==> qmpdclient-1.0.7.tar.gz: qmpdclient-1.0.7/src/tagguesser.cpp <==
/*
 * QMPDClient - An MPD client written in Qt 4.
 * Copyright (C) 2005-2006 Hvard Tautra Knutsen <havtknut@tihlde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "config.h"
#include "tagguesser.h"
#include <QStringList>

// #define DEBUG(A) A
#define DEBUG(A)

QHash<QString, QString> TagGuesser::patterns() {
	QHash<QString, QString> patterns;
	patterns["/%b/%a/%n - %t."] = "/music/Artist/Album/42 - Title of Song.ogg";
	patterns["/%b/%a (%i)/%n - %t."] = "/music/Artist/Album (1992)/42 - Title of Song.ogg";
	patterns["/%b/%n - %a - %t."] = "/music/Artist/42 - Album - Title of Song.ogg";
	patterns["/%n - %b - %a - %t."] = "/music/42 - Artist - Album - Title of Song.ogg";
	return patterns;
}

void TagGuesser::guessTags(MPDSong song) {
	QString pattern = Config::instance()->guessPattern();

	// Check that we have a pattern at all
	if (pattern.isEmpty())
		return;

	// Cant guess from URL's
	if (song.url().contains("://"))
		return;

	// Return if we already have everything we can guess
	if (!(song.title().isEmpty() || song.album().isEmpty() || song.artist().isEmpty() || song.track().isEmpty()))
		return;

	DEBUG(qDebug("Guessing.."));
	// Tokens to expand. %a=album, %b=band, %n=tracknr, %t=title, %i=ignore
	QStringList matchers = pattern.split(QRegExp("%[abnti]"), QString::SkipEmptyParts);
	QStringList markers = pattern.split('%', QString::SkipEmptyParts);

	// Check that we actually have any matchers, if not, return filename as title, the rest empty.

	DEBUG(qDebug("Matchers size: %d %d", matchers.size(), markers.size()));
	if (matchers.isEmpty() || markers.isEmpty()) {

		DEBUG(qDebug("No matchers or no markers"));
		if (song.title().isEmpty())
			song.setTitle(song.url());
		return;
	}

	// Remove bogus markers

	DEBUG(qDebug("Removing bogus markers"));
	markers = markers.filter(QRegExp("^[abnti]"));
	// Remove trailing garbage on markers

	DEBUG(qDebug("Removing trailing garbage on markers"));
	markers = markers.replaceInStrings(QRegExp("^([abnti]).*"), "%\\1");


	DEBUG(qDebug("Adding greedy front matcher"));
	// If pattern does not have leading characters, add a greedy matcher at front so we get the whole start
	if (pattern.contains(QRegExp("^%[abnti]")))
		matchers.push_front("^");

	// If pattern contains trailing characters, add a dummy marker at the end.
	DEBUG(qDebug("Finding last matcher"));
	QString lastMarker = markers.last();

	DEBUG(qDebug("Finding end of last marker"));
	int endOfLastMatcher = pattern.lastIndexOf(lastMarker) + lastMarker.size();

	DEBUG(qDebug("Adding dummy end marker"));
	if (endOfLastMatcher < pattern.size())
		markers += "trailingcharacters";

	DEBUG(foreach(QString p, markers) qDebug("marker : %s", qPrintable(p))
	      foreach(QString p, matchers) qDebug("matcher: %s", qPrintable(p)));

	DEBUG(qDebug("checkpoint"));
	// Match pattern, starting last
	QString left = song.url();

	DEBUG(qDebug("Matching pattern, starting last"));
	while (!matchers.empty()) {
		QString p = matchers.takeLast();
		QString m = markers.takeLast();
		int pos = left.lastIndexOf(p);
		QString result = left.mid(pos + p.size());
		DEBUG(qDebug("%s, %s matches: %s at pos %d", qPrintable(left), qPrintable(m), qPrintable(result), pos));
		left = left.mid(0, pos);
		// Save what we found
		if (m == "%a" && song.album().isEmpty())
			song.setAlbum(result);
		if (m == "%b" && song.artist().isEmpty())
			song.setArtist(result);
		if (m == "%n" && song.track().isEmpty())
			song.setTrack(result);
		if (m == "%t" && song.title().isEmpty())
			song.setTitle(result);
	}
}
